/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg;

import org.springframework.beans.factory.annotation.Autowired;
import java.util.List;
import org.springframework.ai.chat.messages.Message;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.document.Document;
import org.springframework.ai.vectorstore.SearchRequest;
import org.springframework.ai.chat.model.ChatModel;
import org.springframework.ai.chat.model.ChatResponse;
import org.springframework.ai.chat.model.Generation;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.chat.prompt.PromptTemplate;
import org.springframework.ai.chat.prompt.SystemPromptTemplate;
import org.springframework.ai.vectorstore.VectorStore;
import org.springframework.stereotype.Service;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @author omozegieaziegbe
 */
@Service
public class MovieRagService {

    @Autowired
    private VectorStore vectorStore;

    @Autowired
    private ChatModel chatClient;

    private final int topK = 5; // Number of top documents to retrieve

    public Generation retrieve(String message) {
        // Compute the vector of the user query
        SearchRequest searchRequest = SearchRequest.query(message).withTopK(topK);
        // Retrieve the most relevant documents from Redis
        List<Document> documents = vectorStore.similaritySearch(searchRequest);
        Message systemMessage = createPrompt(documents);
        UserMessage userMessage = new UserMessage(message);
        // Construct the prompt for the AI model
        Prompt prompt = new Prompt(List.of(systemMessage, userMessage));
        // Generate a response using the AI model
        ChatResponse response = chatClient.call(prompt);
        return response.getResult();
    }

    private Message createPrompt(List<Document> similarDocuments) {
        String documents = similarDocuments.stream().map(Document::getContent).collect(Collectors.joining("\n"));
        SystemPromptTemplate systemPromptTemplate = new SystemPromptTemplate(PROMPT_TEMPLATE);
        return systemPromptTemplate.createMessage(Map.of("documents", documents));
    }

    private final String PROMPT_TEMPLATE = """
You're assisting with questions about movies in a catalog.
Use the information from the DOCUMENTS section to provide accurate answers.
If the question involves referring to the release year or genre of a movie, include the movie name in the response.
If unsure, simply state that you don't know.

DOCUMENTS:
{documents}
""";
}
